from dataclasses import dataclass, field
from typing import List, Optional, Dict

from dataclasses_json import dataclass_json

from earningscall.event import EarningsEvent


@dataclass_json
@dataclass
class SpeakerInfo:
    name: Optional[str] = field(default=None)
    title: Optional[str] = field(default=None)


@dataclass_json
@dataclass
class Speaker:
    speaker: str
    speaker_info: Optional[SpeakerInfo] = field(default=None)
    text: Optional[str] = field(default=None)
    words: Optional[List[str]] = field(default=None)
    start_times: Optional[List[float]] = field(default=None)


@dataclass_json
@dataclass
class Transcript:
    text: Optional[str] = field(default=None)
    event: Optional[EarningsEvent] = field(default=None)
    speakers: Optional[List[Speaker]] = field(default=None)
    prepared_remarks: Optional[str] = field(default=None)
    questions_and_answers: Optional[str] = field(default=None)
    speaker_name_map_v2: Optional[Dict[str, SpeakerInfo]] = field(default=None)



import datetime

from earningscall.event import EarningsEvent, _parse_conference_date


def test_basic_object_creation():
    event = EarningsEvent(
        year=2024,
        quarter=1,
        conference_date=datetime.datetime.fromisoformat("2024-04-28T15:00:00.000-05:00"),
    )
    event.to_json()


def test_basic_without_conference_date():
    event = EarningsEvent(
        year=2024,
        quarter=1,
    )
    raw_json = event.to_json()
    event_after = EarningsEvent.from_json(raw_json)
    assert event_after.conference_date is None
    assert event_after.year == 2024
    assert event_after.quarter == 1


def test_date_field_deserialization():
    #
    earnings_event = EarningsEvent.from_dict(
        {
            "year": 2024,
            "quarter": 1,
            "conference_date": "2024-04-28T15:00:00.000-05:00",
        }
    )
    assert earnings_event.year == 2024
    assert earnings_event.quarter == 1
    assert earnings_event.conference_date == datetime.datetime.fromisoformat("2024-04-28T15:00:00.000-05:00")


def test_date_field_deserialization_with_z_suffix():
    """Test that the date parser correctly handles 'Z' UTC suffix"""
    earnings_event = EarningsEvent.from_dict(
        {
            "year": 2021,
            "quarter": 1,
            "conference_date": "2021-01-25T13:30:00.000Z",
        }
    )
    assert earnings_event.year == 2021
    assert earnings_event.quarter == 1
    expected_date = datetime.datetime.fromisoformat("2021-01-25T13:30:00.000+00:00")
    assert earnings_event.conference_date == expected_date


def test_parse_conference_date_with_none():
    """Test that _parse_conference_date properly handles None input"""
    result = _parse_conference_date(None)
    assert result is None


def test_earnings_event_from_dict_with_none_conference_date():
    """Test that EarningsEvent.from_dict works when conference_date is None"""
    earnings_event = EarningsEvent.from_dict(
        {
            "year": 2024,
            "quarter": 2,
            "conference_date": None,
        }
    )
    assert earnings_event.year == 2024
    assert earnings_event.quarter == 2
    assert earnings_event.conference_date is None



import logging
from typing import Optional, List

import requests

from earningscall import api
from earningscall.errors import InsufficientApiAccessError
from earningscall.event import EarningsEvent
from earningscall.symbols import CompanyInfo
from earningscall.transcript import Transcript

log = logging.getLogger(__file__)


class Company:
    """
    A class representing a company.

    Can be used to retrieve events, transcripts, and audio files.
    """

    company_info: CompanyInfo
    name: Optional[str]
    _events: Optional[List[EarningsEvent]]

    def __init__(self, company_info: CompanyInfo):
        if not company_info:
            raise ValueError("company_info must be present.")
        self.company_info = company_info
        self.name = company_info.name
        self._events = None

    def __str__(self):
        return str(self.name)

    def _get_events(self) -> List[EarningsEvent]:
        if not self.company_info.exchange or not self.company_info.symbol:
            return []
        raw_response = api.get_events(self.company_info.exchange, self.company_info.symbol)
        if not raw_response:
            return []
        return [EarningsEvent.from_dict(event) for event in raw_response["events"]]  # type: ignore

    def events(self) -> List[EarningsEvent]:
        if not self._events:
            self._events = self._get_events()
        return self._events

    def get_transcript(
        self,
        year: Optional[int] = None,
        quarter: Optional[int] = None,
        event: Optional[EarningsEvent] = None,
        level: Optional[int] = None,
    ) -> Optional[Transcript]:
        """
        Get the transcript for a given year and quarter.

        :param Optional[int] year: The year to get the transcript for.
        :param Optional[int] quarter: The quarter to get the transcript for.
        :param Optional[EarningsEvent] event: The event to get the transcript for.
        :param Optional[int] level: The transcript level to retrieve.  Default: 1

        :return: The transcript for the given year and quarter.
        """
        if not self.company_info.exchange or not self.company_info.symbol:
            return None
        if (not year or not quarter) and event:
            year = event.year
            quarter = event.quarter
        if not year or not quarter:
            raise ValueError("Must specify either event or year and quarter")
        if quarter < 1 or quarter > 4:
            raise ValueError("Invalid level. Must be one of: {1,2,3,4}")
        if level is None:
            level = 1
        if type(level) is not int or level <= 0 or level > 4:
            raise ValueError(f"Invalid level: {level}. Must be one of: 1, 2, 3, or 4.")
        try:
            response_payload = api.get_transcript(
                self.company_info.exchange,
                self.company_info.symbol,
                year,
                quarter,
                level=level,
            )
            # TODO: Investigate alternatives to @dataclass for level 3 transcripts, as this is
            #       extremely slow.
            transcript = Transcript.from_dict(response_payload)  # type: ignore
            if level == 3:
                for speaker in transcript.speakers:
                    speaker.text = " ".join(speaker.words)
            if 2 <= level <= 3:
                transcript.text = " ".join(map(lambda spk: spk.text, transcript.speakers))
            elif level == 4:
                if transcript.questions_and_answers:
                    transcript.text = " ".join([transcript.prepared_remarks, transcript.questions_and_answers])
                else:
                    transcript.text = transcript.prepared_remarks
            if transcript.speaker_name_map_v2:
                for speaker in transcript.speakers:
                    speaker.speaker_info = transcript.speaker_name_map_v2.get(speaker.speaker)
            return transcript
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                return None
            if error.response.status_code == 403:
                plan_name = error.response.headers.get("X-Plan-Name", "unknown")
                if 2 <= level <= 4:
                    error_message = (
                        f"Your plan ({plan_name}) does not include Advanced Transcript Data. "
                        "Upgrade your plan here: https://earningscall.biz/api-pricing"
                    )
                else:
                    error_message = f"Unexpected error code was returned from the server.  Your plan is: {plan_name}"
                log.error(error_message)
                raise InsufficientApiAccessError(error_message)
            raise error

    def download_audio_file(
        self,
        year: Optional[int] = None,
        quarter: Optional[int] = None,
        event: Optional[EarningsEvent] = None,
        file_name: Optional[str] = None,
    ) -> Optional[str]:
        """
        Download the audio file for a given year and quarter.

        :param Optional[int] year: The year to get the audio for.
        :param Optional[int] quarter: The quarter to get the audio for.
        :param Optional[EarningsEvent] event: The event to get the audio for.
        :param Optional[str] file_name: The file name to save the audio to.

        :return: The audio for the given year and quarter.
        """
        if not self.company_info.exchange or not self.company_info.symbol:
            return None
        if (not year or not quarter) and event:
            year = event.year
            quarter = event.quarter
        if not year or not quarter:
            raise ValueError("Must specify either event or year and quarter")
        if quarter < 1 or quarter > 4:
            raise ValueError("Invalid level. Must be one of: {1,2,3,4}")
        try:
            resp = api.download_audio_file(
                exchange=self.company_info.exchange,
                symbol=self.company_info.symbol,
                year=year,
                quarter=quarter,
                file_name=file_name,
            )
            return resp
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                return None
            if error.response.status_code == 403:
                plan_name = error.response.headers["X-Plan-Name"]
                error_message = (
                    f"Your plan ({plan_name}) does not include Audio Files. "
                    "Upgrade your plan here: https://earningscall.biz/api-pricing"
                )
                log.error(error_message)
                raise InsufficientApiAccessError(error_message)
            raise error

    def download_slide_deck(
        self,
        year: Optional[int] = None,
        quarter: Optional[int] = None,
        event: Optional[EarningsEvent] = None,
        file_name: Optional[str] = None,
    ) -> Optional[str]:
        """
        Download the slide deck for a given year and quarter.

        :param Optional[int] year: The year to get the slide deck for.
        :param Optional[int] quarter: The quarter to get the slide deck for.
        :param Optional[EarningsEvent] event: The event to get the slide deck for.
        :param Optional[str] file_name: The file name to save the slide deck to.

        :return: The filename of the downloaded slide deck file.
        """
        if not self.company_info.exchange or not self.company_info.symbol:
            return None
        if (not year or not quarter) and event:
            year = event.year
            quarter = event.quarter
        if not year or not quarter:
            raise ValueError("Must specify either event or year and quarter")
        if quarter < 1 or quarter > 4:
            raise ValueError("Invalid quarter. Must be one of: {1,2,3,4}")
        try:
            resp = api.download_slide_deck(
                exchange=self.company_info.exchange,
                symbol=self.company_info.symbol,
                year=year,
                quarter=quarter,
                file_name=file_name,
            )
            return resp
        except requests.exceptions.HTTPError as error:
            if error.response.status_code == 404:
                return None
            if error.response.status_code == 403:
                plan_name = error.response.headers["X-Plan-Name"]
                error_message = (
                    f"Your plan ({plan_name}) does not include Slide Decks. "
                    "Upgrade your plan here: https://earningscall.biz/api-pricing"
                )
                log.error(error_message)
                raise InsufficientApiAccessError(error_message)
            raise error



from datetime import datetime, date, timedelta

import pytest
import requests
import responses

import earningscall
from earningscall import get_calendar
from earningscall.api import purge_cache
from earningscall.errors import InsufficientApiAccessError
from earningscall.utils import data_path


# Uncomment and run following code to generate data/get-calendar-not-found-response.yaml file
#

# import earningscall
# @_recorder.record(file_path="data/get-calendar-not-found-response.yaml")
# def test_save_symbols_v1():
#     requests.get("https://v2.api.earningscall.biz/calendar?apikey=demo&year=1980&month=1&day=1")


@pytest.fixture(autouse=True)
def run_before_and_after_tests():
    """Fixture to execute asserts before and after a test is run"""
    # Setup
    earningscall.api_key = None
    earningscall.retry_strategy = {
        "strategy": "exponential",
        "base_delay": 0.01,
        "max_attempts": 3,
    }
    purge_cache()
    yield  # this is where the testing happens
    # Teardown
    earningscall.api_key = None


@responses.activate
def test_get_calendar_invalid_inputs():
    with pytest.raises(ValueError):
        get_calendar(None)
    with pytest.raises(ValueError):
        get_calendar("2025-02-10")
    with pytest.raises(ValueError):
        get_calendar(123456)
    with pytest.raises(ValueError):
        get_calendar(date(2017, 12, 31))
    with pytest.raises(ValueError):
        get_calendar(datetime.now() + timedelta(days=31))


@responses.activate
def test_get_non_demo_date():
    with pytest.raises(InsufficientApiAccessError):
        get_calendar(date(2020, 1, 1))


@responses.activate
def test_get_calendar_success():
    earningscall.api_key = "XXXXXXXXXXX"
    responses._add_from_file(file_path=data_path("get-calendar-successful-response.yaml"))
    calendar = get_calendar(date(2025, 2, 10))
    assert len(calendar) == 20
    assert calendar[0].exchange == "NASDAQ"
    assert calendar[0].symbol == "MPAA"
    assert calendar[0].year == 2025
    assert calendar[0].quarter == 3
    assert calendar[0].conference_date.year == 2025
    assert calendar[0].conference_date.month == 2
    assert calendar[0].conference_date.day == 10
    assert calendar[0].transcript_ready
    assert calendar[0].company_name == "Motorcar Parts of America, Inc."


@responses.activate
def test_get_calendar_success_with_datetime():
    earningscall.api_key = "XXXXXXXXXXX"
    responses._add_from_file(file_path=data_path("get-calendar-successful-response.yaml"))
    calendar = get_calendar(datetime(2025, 2, 10))
    assert len(calendar) == 20
    assert calendar[0].exchange == "NASDAQ"
    assert calendar[0].symbol == "MPAA"
    assert calendar[0].year == 2025
    assert calendar[0].quarter == 3
    assert calendar[0].conference_date.year == 2025
    assert calendar[0].conference_date.month == 2
    assert calendar[0].conference_date.day == 10
    assert calendar[0].transcript_ready
    assert calendar[0].company_name == "Motorcar Parts of America, Inc."


@responses.activate
def test_get_calendar_server_error():
    earningscall.api_key = "XXXXXXXXXXX"
    responses._add_from_file(file_path=data_path("get-calendar-500-error.yaml"))
    with pytest.raises(requests.exceptions.HTTPError):
        get_calendar(date(2020, 1, 1))


@responses.activate
def test_get_calendar_not_found():
    earningscall.api_key = "XXXXXXXXXXX"
    responses._add_from_file(file_path=data_path("get-calendar-not-found-response.yaml"))
    calendar = get_calendar(date(2018, 1, 1))
    assert len(calendar) == 0



import earningscall  # noqa: F401

from earningscall import get_company


# TODO: Set your API key here:
# earningscall.api_key = "YOUR API KEY HERE"

company = get_company("AAPL")

transcript = company.get_transcript(year=2021, quarter=3, level=2)

speaker = transcript.speakers[1]
speaker_label = speaker.speaker_info.name
text = speaker.text
print("Speaker:")
print(f"  Name: {speaker.speaker_info.name}")
print(f"  Title: {speaker.speaker_info.title}")
print()
print(f"Text: {text}")



import time

import responses

from earningscall.api import get_sp500_companies_txt_file, cached_urls, purge_cache


@responses.activate
def test_load_sp500_tickers():
    purge_cache()
    assert len(cached_urls()) == 0
    ##
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo",
        body="AAPL\nMSFT\nTSLA",
        status=200,
        adding_headers={
            "Cache-Control": "public, max-age=3",  # Allow client to cache for 3 seconds
        },
    )
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo",
        body="AAPL\nMSFT\nTSLA\nNEWCOMPANY",
        status=200,
        adding_headers={
            "cache-control": "public, max-age=30",  # Allow client to cache for 30 seconds
        },
    )
    ##
    sp500_raw_text = get_sp500_companies_txt_file()
    ##
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == "https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"
    assert cached_urls() == ["https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"]
    tickers = [ticker_symbol for ticker_symbol in sp500_raw_text.split("\n")]
    assert tickers == ["AAPL", "MSFT", "TSLA"]
    ##
    # Request a second time, should serve out of cache
    ##
    sp500_raw_text = get_sp500_companies_txt_file()
    ##
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == "https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"
    urls = cached_urls()
    assert urls == ["https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"]
    tickers = [ticker_symbol for ticker_symbol in sp500_raw_text.split("\n")]
    assert tickers == ["AAPL", "MSFT", "TSLA"]
    ##
    # Wait enough time for cache entry to expire
    ##
    time.sleep(4)
    ##
    sp500_raw_text = get_sp500_companies_txt_file()
    ##
    assert len(responses.calls) == 2
    assert responses.calls[0].request.url == "https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"
    assert responses.calls[1].request.url == "https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"
    assert cached_urls() == ["https://v2.api.earningscall.biz/symbols/sp500.txt?apikey=demo"]
    tickers = [ticker_symbol for ticker_symbol in sp500_raw_text.split("\n")]
    assert tickers == ["AAPL", "MSFT", "TSLA", "NEWCOMPANY"]



import argparse
from datetime import datetime
import os

import earningscall  # noqa: F401
from earningscall import get_company
from earningscall.company import Company
from earningscall.utils import configure_sane_logging


# TODO: Set your API key here:
# earningscall.api_key = "YOUR SECRET API KEY GOES HERE"


parser = argparse.ArgumentParser(description='')
parser.add_argument('--debug', action='store_true', help='Enable debug logs')
parser.add_argument('--sp-500', action='store_true', help='Show S&P500 Companies')

args = parser.parse_args()
configure_sane_logging()


directory = "audio_files"
os.makedirs(directory, exist_ok=True)


def download_audio_files(company: Company):
    print(f"Downloading all audio files for: {company}..")
    for event in company.events():
        if datetime.now().timestamp() < event.conference_date.timestamp():
            print(
                f"* {company.company_info.symbol} Q{event.quarter} {event.year} -- skipping, conference date in the future"
            )
            continue
        file_name = os.path.join(
            directory,
            f"{company.company_info.exchange}_{company.company_info.symbol}_{event.year}_Q{event.quarter}.mp3",
        )
        if os.path.exists(file_name):
            print(f"* {company.company_info.symbol} Q{event.quarter} {event.year} -- already downloaded")
        else:
            print(f"* Downloading audio file for {company.company_info.symbol} Q{event.quarter} {event.year}...")
            audio_file = company.download_audio_file(event=event, file_name=file_name)
            if audio_file:
                print(f" Downloaded audio file: \"{audio_file}\"")
            else:
                print(f" No audio file found for {company.company_info.symbol} Q{event.quarter} {event.year}")


company = get_company("aapl")  # Lookup Apple, Inc by its ticker symbol, "AAPL"
download_audio_files(company)



import os

import pytest
import responses
from requests import HTTPError

import earningscall
from earningscall import get_company
from earningscall.api import purge_cache
from earningscall.company import Company
from earningscall.errors import InsufficientApiAccessError
from earningscall.event import EarningsEvent
from earningscall.symbols import CompanyInfo, clear_symbols
from earningscall.utils import data_path


@pytest.fixture(autouse=True)
def run_before_and_after_tests():
    """Fixture to execute asserts before and after a test is run"""
    # Setup: fill with any logic you want
    earningscall.api_key = None
    earningscall.retry_strategy = {
        "strategy": "exponential",
        "base_delay": 0.001,
        "max_attempts": 3,
    }
    purge_cache()
    clear_symbols()
    yield  # this is where the testing happens
    # Teardown : fill with any logic you want
    earningscall.api_key = None


@responses.activate
def test_download_slide_deck():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("msft-q1-2022-slide-deck-short-clip.yaml"))
    responses._add_from_file(file_path=data_path("msft-company-events.yaml"))
    ##
    company = get_company("msft")
    file_name = company.download_slide_deck(year=2022, quarter=1)
    #
    assert os.path.exists(file_name)
    assert os.path.getsize(file_name) > 0
    # Check that it's a PDF file (starts with %PDF)
    with open(file_name, 'rb') as f:
        assert f.read(4) == b'%PDF'
    os.unlink(file_name)


@responses.activate
def test_download_slide_deck_event():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("msft-q1-2022-slide-deck-short-clip.yaml"))
    responses._add_from_file(file_path=data_path("msft-company-events.yaml"))
    ##
    company = get_company("msft")
    file_name = company.download_slide_deck(event=EarningsEvent(year=2022, quarter=1))
    #
    assert os.path.exists(file_name)
    assert os.path.getsize(file_name) > 0
    # Check that it's a PDF file (starts with %PDF)
    with open(file_name, 'rb') as f:
        assert f.read(4) == b'%PDF'
    os.unlink(file_name)


@responses.activate
def test_download_slide_deck_missing_params_raises_value_error():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    ##
    company = get_company("msft")
    with pytest.raises(ValueError):
        company.download_slide_deck()
    with pytest.raises(ValueError):
        company.download_slide_deck(year=2023)
    with pytest.raises(ValueError):
        company.download_slide_deck(quarter=1)
    with pytest.raises(ValueError):
        company.download_slide_deck(quarter=0)
    with pytest.raises(ValueError):
        company.download_slide_deck(year=2023, quarter=5)
    ##
    invalid_company = Company(company_info=CompanyInfo())
    slide_deck = invalid_company.download_slide_deck(year=2023, quarter=1)
    ##
    assert slide_deck is None


@responses.activate
def test_download_slide_deck_missing_from_server():
    ##
    earningscall.api_key = "foobar"  # Set to a bogus API Key.
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("meta-q3-2024-slides-not-found.yaml"))
    ##
    company = get_company("meta")
    output_file = company.download_slide_deck(year=2024, quarter=3)
    ##
    assert output_file is None


@responses.activate
def test_download_slide_deck_not_authorized():
    ##
    earningscall.api_key = "foobar"  # Set to a bogus API Key.
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("meta-q3-2024-slides-not-authorized.yaml"))
    ##
    company = get_company("meta")
    with pytest.raises(InsufficientApiAccessError):
        company.download_slide_deck(year=2024, quarter=3)


@responses.activate
def test_download_slide_deck_500_error():
    ##
    earningscall.api_key = "foobar"  # Set to a bogus API Key.
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("meta-q3-2024-slides-other-error.yaml"))
    ##
    company = get_company("meta")
    with pytest.raises(HTTPError):
        company.download_slide_deck(year=2024, quarter=3)


@responses.activate
def test_download_slide_deck_custom_filename():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("msft-q1-2022-slide-deck-short-clip.yaml"))
    responses._add_from_file(file_path=data_path("msft-company-events.yaml"))
    ##
    company = get_company("msft")
    custom_filename = "test_custom_slides.pdf"
    file_name = company.download_slide_deck(year=2022, quarter=1, file_name=custom_filename)
    #
    assert file_name == custom_filename
    assert os.path.exists(file_name)
    assert os.path.getsize(file_name) > 0
    # Check that it's a PDF file (starts with %PDF)
    with open(file_name, 'rb') as f:
        assert f.read(4) == b'%PDF'
    os.unlink(file_name)



import earningscall  # noqa: F401
from earningscall import get_company

# earningscall.api_key = "YOUR API KEY HERE"


company = get_company("AAPL")  # Lookup Apple, Inc by its ticker symbol, "AAPL"

print(f"Getting all transcripts for: {company}..")

for event in company.events():
    print(f"* Q{event.quarter} {event.year}")
    transcript = company.get_transcript(event=event)
    if transcript:
        print(f"  Transcript Text: \"{transcript.text[:100]}...\"")
    else:
        print("  No transcript found.")



import datetime
from datetime import date, timedelta
from typing import List, Optional, Iterator

import requests

from earningscall import api
from earningscall.api import get_sp500_companies_txt_file, is_demo_account
from earningscall.calendar import CalendarEvent
from earningscall.company import Company
from earningscall.errors import InsufficientApiAccessError
from earningscall.symbols import get_symbols


def get_company(symbol: str, exchange: Optional[str] = None) -> Optional[Company]:
    """
    Get a company by symbol and optionally an exchange.

    :param str symbol: The symbol to get the company for.
    :param Optional[str] exchange: The exchange to get the company for.

    :return: The company for the given symbol and exchange.
    """
    company_info = get_symbols().lookup_company(symbol=symbol, exchange=exchange)
    if company_info:
        return Company(company_info=company_info)
    return None


def get_all_companies() -> Iterator[Company]:
    """
    Get all companies.

    :return: An iterator of all companies that is available to the current API plan.
    """
    for company_info in get_symbols().get_all():
        yield Company(company_info=company_info)


def get_sp500_companies() -> Iterator[Company]:
    """
    Get all S&P 500 companies.

    :return: An iterator of all S&P 500 companies that is available to the current API plan.
    """
    resp = get_sp500_companies_txt_file()
    if not resp:
        return
    for ticker_symbol in resp.split("\n"):
        company_info = get_symbols().lookup_company(ticker_symbol)
        if company_info:
            yield Company(company_info=company_info)


def get_calendar(input_date: date) -> List[CalendarEvent]:
    """
    Get the earnings event calendar for a given input date.

    :param date input_date: The date to get the calendar for.

    :return: A list of CalendarEvent objects.
    """
    if not input_date:
        raise ValueError("Date is required")
    if isinstance(input_date, datetime.datetime):
        input_date = input_date.date()
    if not isinstance(input_date, date):
        raise ValueError("Date must be a date object")
    if input_date < date(2018, 1, 1):
        raise ValueError("input_date must be greater than or equal to 2018-01-01")
    # Check if input_date is greater than 30 days from today
    if input_date > datetime.datetime.now().date() + timedelta(days=30):
        raise ValueError("input_date must be less than 30 days from today")
    if is_demo_account() and input_date != date(2025, 1, 10):
        raise InsufficientApiAccessError(
            f"\"{input_date}\" requires an API Key for access.  To get your API Key,"
            f" see: https://earningscall.biz/api-pricing"
        )
    try:
        json_data = api.get_calendar_api_operation(input_date.year, input_date.month, input_date.day)
        return [CalendarEvent.from_dict(event) for event in json_data]  # type: ignore
    except requests.exceptions.HTTPError as error:
        if error.response.status_code == 404:
            # Calendar Date not found, simply return an empty list
            return []
        raise error



from earningscall.errors import InsufficientApiAccessError


def test_errors():
    ##
    error = InsufficientApiAccessError("You don't have access")
    ##
    assert error.msg == "You don't have access"
    assert str(error) == "You don't have access"
    ##
    error = InsufficientApiAccessError()
    ##
    assert error.msg is None
    assert str(error) == ""



import json

import pytest

from earningscall.company import Company
from earningscall.symbols import CompanyInfo


def test_company_constructor_checks():
    with pytest.raises(ValueError):
        Company(company_info=None)


def test_company_to_string():
    company_info = CompanyInfo(
        name="Test Company",
        symbol="TEST",
        exchange="TEST",
        sector="TEST",
        industry="TEST",
    )
    company = Company(company_info=company_info)
    assert str(company) == "Test Company"
    assert str(company.company_info) == "(TEST: TEST - Test Company)"
    assert company.company_info == company_info
    assert json.loads(company_info.to_json()) == {
        "name": "Test Company",
        "symbol": "TEST",
        "exchange": "TEST",
        "sector": "TEST",
        "industry": "TEST",
    }
    assert company_info.exchange_symbol() == "TEST_TEST"
    assert company_info.to_txt_row() == ["-1", "TEST", "Test Company"]



import logging
from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from dataclasses_json import config
from dataclasses_json import dataclass_json
from marshmallow import fields

log = logging.getLogger(__file__)


def _parse_conference_date(date_str: str) -> Optional[datetime]:
    """Parse conference date string, handling both timezone offsets and 'Z' UTC suffix."""
    if not date_str:
        return None
    # Handle 'Z' suffix by converting to '+00:00' for UTC
    if date_str.endswith('Z'):
        date_str = date_str[:-1] + '+00:00'
    return datetime.fromisoformat(date_str)


@dataclass_json
@dataclass
class EarningsEvent:
    """
    EarningsEvent
    """

    year: int
    quarter: int
    conference_date: Optional[datetime] = field(
        default=None,
        metadata=config(
            encoder=lambda date: date.isoformat() if date else None,
            decoder=lambda date: _parse_conference_date(date),
            mm_field=fields.DateTime(format="iso"),
        ),
    )



import json

import pytest
import responses
from requests import HTTPError

import earningscall
from earningscall import get_company
from earningscall.api import purge_cache
from earningscall.company import Company
from earningscall.errors import InsufficientApiAccessError, InvalidApiKeyError
from earningscall.event import EarningsEvent
from earningscall.symbols import clear_symbols, CompanyInfo
from earningscall.transcript import Transcript
from earningscall.utils import data_path


# Uncomment and run following code to generate msft-transcript-response.yaml file
#
# from responses import _recorder
# @_recorder.record(file_path="msft-transcript-response.yaml")
# def test_save_symbols_v1():
#     requests.get("https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=MSFT&year=2023&quarter=1")


@pytest.fixture(autouse=True)
def run_before_and_after_tests():
    """Fixture to execute asserts before and after a test is run"""
    # Setup: fill with any logic you want
    earningscall.api_key = None
    earningscall.retry_strategy = {
        "strategy": "exponential",
        "base_delay": 0.01,
        "max_attempts": 3,
    }
    purge_cache()
    clear_symbols()
    yield  # this is where the testing happens
    # Teardown : fill with any logic you want
    earningscall.api_key = None


@responses.activate
def test_get_transcript_invalid_inputs():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("msft-transcript-response.yaml"))
    ##
    company = get_company("msft")
    ##
    with pytest.raises(ValueError):
        company.get_transcript(year=2023)
    with pytest.raises(ValueError):
        company.get_transcript(quarter=1)
    with pytest.raises(ValueError):
        company.get_transcript(quarter=0)
    with pytest.raises(ValueError):
        company.get_transcript(year=2023, quarter=5)
    with pytest.raises(ValueError):
        company.get_transcript(year=2023, quarter=1, level=0)
    with pytest.raises(ValueError):
        company.get_transcript(year=2023, quarter=1, level=5)
    ##
    invalid_company = Company(company_info=CompanyInfo())
    transcript = invalid_company.get_transcript(year=2023, quarter=1)
    ##
    assert transcript is None


@responses.activate
def test_get_demo_company():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("msft-transcript-response.yaml"))
    ##
    company = get_company("msft")
    ##
    transcript = company.get_transcript(year=2023, quarter=1)
    assert transcript.text[:100] == (
        'Greetings, and welcome to the Microsoft Fiscal Year 2023 First Quarter Earnings ' 'Conference Call. At '
    )


@responses.activate
def test_get_demo_company_with_event_populated():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("demo-symbols-v2-alpha.yaml"))
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(event=EarningsEvent(year=2022, quarter=1))
    assert transcript.event.year == 2022
    assert transcript.event.quarter == 1
    assert transcript.event.conference_date.isoformat() == "2022-01-19T00:00:00-08:00"
    assert transcript.text[:100] == (
        "Good day and welcome to the Apple Q1 Fiscal Year 2021 earnings conference " "call. Today's call is bein"
    )


@responses.activate
def test_get_demo_company_with_advanced_transcript_data():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2022-advanced-data-level-2.yaml"))
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(year=2022, quarter=1, level=2)
    ##
    assert transcript.event.year == 2022
    assert transcript.event.quarter == 1
    assert transcript.event.conference_date.isoformat() == "2022-01-27T14:00:00-08:00"
    assert transcript.text[:100] == (
        "Good day and welcome to the Apple Q1 FY 2022 earnings conference call. Today's call is being recorde"
    )
    assert transcript.speakers[0].speaker == "spk03"
    assert transcript.speakers[0].words is None
    assert transcript.speakers[0].start_times is None


@responses.activate
def test_get_demo_company_with_speaker_name_map_v2():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2022-speaker-name-map-v2.yaml"))
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(year=2023, quarter=1, level=2)
    ##
    assert transcript.event.year == 2023
    assert transcript.event.quarter == 1
    assert transcript.event.conference_date.isoformat() == "2023-02-02T17:00:00-05:00"
    assert transcript.text[:100] == (
        "Good day, everyone, and welcome to the Apple Q1 Fiscal Year 2023 Earnings Conference Call. Today's c"
    )
    assert transcript.speakers[0].speaker == "spk12"
    assert transcript.speakers[0].speaker_info.name == "Teja Skala"
    assert transcript.speakers[0].speaker_info.title == "Director of Investor Relations and Corporate Finance"
    assert transcript.speakers[0].words is None
    assert transcript.speakers[0].start_times is None
    assert transcript.speakers[1].speaker == "spk07"
    assert transcript.speakers[1].speaker_info.name == "Operator"
    assert transcript.speakers[1].speaker_info.title is None
    assert transcript.speakers[1].words is None
    assert transcript.speakers[1].start_times is None
    assert transcript.speaker_name_map_v2["spk01"].name == "Tim Cook"
    assert transcript.speaker_name_map_v2["spk01"].title == "CEO"


@responses.activate
def test_get_demo_company_with_advanced_transcript_data_level_3():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2022-advanced-data-level-3.yaml"))
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(year=2022, quarter=1, level=3)
    ##
    assert transcript.event.year == 2022
    assert transcript.event.quarter == 1
    assert transcript.event.conference_date.isoformat() == "2022-01-27T14:00:00-08:00"
    assert transcript.text[:100] == (
        "Good day and welcome to the Apple Q1 FY 2022 earnings conference call. Today's call is being recorde"
    )
    assert transcript.speakers[0].speaker == "spk03"
    assert " ".join(transcript.speakers[0].words)[:100] == (
        "Good day and welcome to the Apple Q1 FY 2022 earnings conference call. Today's call is being recorde"
    )
    assert transcript.speakers[0].start_times[:7] == [
        0.029,
        0.189,
        0.429,
        0.57,
        0.91,
        1.03,
        1.27,
    ]


@responses.activate
def test_get_demo_company_with_advanced_transcript_data_level_4():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2022-advanced-data-level-4.yaml"))
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(year=2022, quarter=1, level=4)
    ##
    assert transcript.event.year == 2022
    assert transcript.event.quarter == 1
    assert transcript.event.conference_date.isoformat() == "2022-01-27T14:00:00-08:00"
    assert transcript.text[:100] == (
        "Good day and welcome to the Apple Q1 FY 2022 earnings conference call. Today's call is being recorde"
    )
    assert transcript.prepared_remarks[:100] == (
        "Good day and welcome to the Apple Q1 FY 2022 earnings conference call. Today's call is being recorde"
    )
    assert transcript.questions_and_answers[:100] == (
        "We'll take our first question from Katie Huberty with Morgan Stanley. Caller, please check your mute"
    )

    assert transcript.speakers is None


@responses.activate
def test_get_demo_company_with_advanced_transcript_data_level_4_q_and_a_is_missing():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("nvda-q2-2025-level-4-data-missing.yaml"))
    ##
    company = get_company("nvda")
    ##
    transcript = company.get_transcript(year=2025, quarter=2, level=4)
    ##
    assert transcript.event.year == 2025
    assert transcript.event.quarter == 2
    assert transcript.event.conference_date.isoformat() == "2024-08-28T17:00:00-04:00"
    assert transcript.text[:100] == (
        "Good afternoon, everyone, and welcome to NVIDIA's conference call for the second quarter of fiscal 2"
    )
    assert transcript.prepared_remarks[:100] == (
        "Good afternoon, everyone, and welcome to NVIDIA's conference call for the second quarter of fiscal 2"
    )
    assert transcript.questions_and_answers is None
    assert transcript.speakers is None


@responses.activate
def test_get_non_demo_company():
    ##
    responses._add_from_file(file_path=data_path("demo-symbols-v2.yaml"))
    ##
    with pytest.raises(InsufficientApiAccessError):
        get_company("nvda")


@responses.activate
def test_get_transcript_not_found():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2030-not-found.yaml"))
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(year=2030, quarter=1)
    ##
    assert transcript is None


@responses.activate
def test_get_transcript_not_authorized_level_1():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2030-not-authorized.yaml"))
    ##
    company = get_company("aapl")
    ##
    with pytest.raises(InsufficientApiAccessError):
        company.get_transcript(year=2030, quarter=1)


@responses.activate
def test_get_transcript_not_authorized_level_2():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2030-not-authorized-l2.yaml"))
    ##
    company = get_company("aapl")
    ##
    with pytest.raises(InsufficientApiAccessError):
        company.get_transcript(year=2030, quarter=1, level=2)


@responses.activate
def test_get_transcript_server_error():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("aapl-q1-2030-server-error.yaml"))
    ##
    company = get_company("aapl")
    ##
    with pytest.raises(HTTPError):
        company.get_transcript(year=2030, quarter=1)


def test_data_class_for_transcript():
    ##
    transcript = Transcript.from_dict(
        {
            "speakers": [],
            "speaker_name_map_v2": {
                "spk01": {
                    "name": "John Doe",
                    "title": "CEO",
                }
            },
        }
    )
    ##
    assert transcript.speaker_name_map_v2["spk01"].name == "John Doe"
    assert transcript.speaker_name_map_v2["spk01"].title == "CEO"


@responses.activate
def test_get_transcript_with_rate_limit_backoff_and_retry():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    # First response
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=AAPL&year=2023&quarter=1&level=1",
        body=json.dumps({"error": "Rate limit exceeded"}),
        status=429,
    )
    # Second response
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=AAPL&year=2023&quarter=1&level=1",
        body=json.dumps({"text": "Hello, world!"}),
        status=200,
    )
    ##
    company = get_company("aapl")
    ##
    transcript = company.get_transcript(year=2023, quarter=1, level=1)
    # ##
    assert transcript.text == "Hello, world!"


@responses.activate
def test_get_transcript_fails_all_attempts():
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    # Always throttle the caller
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=AAPL&year=2023&quarter=1&level=1",
        body=json.dumps({"error": "Rate limit exceeded"}),
        status=429,
    )
    ##
    company = get_company("aapl")
    ##
    with pytest.raises(HTTPError):
        company.get_transcript(year=2023, quarter=1, level=1)


@responses.activate
def test_get_transcript_fails_all_attempts_linear_retry_strategy():
    earningscall.retry_strategy = {
        "strategy": "linear",
        "base_delay": 0.01,
        "max_attempts": 3,
    }
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    # Always throttle the caller
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=AAPL&year=2023&quarter=1&level=1",
        body=json.dumps({"error": "Rate limit exceeded"}),
        status=429,
    )
    ##
    company = get_company("aapl")
    ##
    with pytest.raises(HTTPError):
        company.get_transcript(year=2023, quarter=1, level=1)


@responses.activate
def test_get_transcript_fails_all_attempts_invalid_retry_strategy():
    earningscall.retry_strategy = {
        "strategy": "invalid",
        "base_delay": 0.01,
        "max_attempts": 3,
    }
    ##
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    # Always throttle the caller
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=AAPL&year=2023&quarter=1&level=1",
        body=json.dumps({"error": "Rate limit exceeded"}),
        status=429,
    )
    ##
    company = get_company("aapl")
    ##
    with pytest.raises(ValueError):
        company.get_transcript(year=2023, quarter=1, level=1)


@responses.activate
def test_get_company_fails_not_authorized():
    # Always throttle the caller
    responses.add(
        responses.GET,
        "https://v2.api.earningscall.biz/symbols-v2.txt",
        body=json.dumps({"error": "Not authorized"}),
        status=401,
    )
    ##
    with pytest.raises(InvalidApiKeyError):
        get_company("aapl")


# Uncomment and run following code to generate demo-symbols-v2.yaml file
#
# import requests
#
# from responses import _recorder
#
#
# @_recorder.record(file_path="data/nvda-q2-2025-level-4-data-missing.yaml")
# def test_save_symbols_v1_first():
#     requests.get("https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=NVDA&year=2025&quarter=2&level=4")

# Uncomment and run following code to generate demo-symbols-v2.yaml file

# import requests
#
# from responses import _recorder
#
#
# @_recorder.record(file_path="data/aapl-q1-2030-not-found.yaml")
# def test_save_symbols_v1_second():
#     requests.get(
#         "https://v2.api.earningscall.biz/transcript?apikey=demo&exchange=NASDAQ&symbol=AAPL&year=2030&quarter=1&level=1"
#     )



from earningscall.utils import configure_sane_logging, enable_debug_logs, project_file_path


def test_project_file_path_tests_dir():
    project_file_path("tests", None)


def test_enable_debug_logs():
    enable_debug_logs()


def test_configure_sane_logging():
    configure_sane_logging()



import responses

from earningscall import get_company
from earningscall.api import purge_cache
from earningscall.symbols import clear_symbols
from earningscall.utils import data_path


# Uncomment and run following code to generate msft-transcript-response.yaml file
#
# from responses import _recorder
# @_recorder.record(file_path="msft-company-events.yaml")
# def test_save_symbols_v1():
#     requests.get("https://v2.api.earningscall.biz/events?apikey=demo&exchange=NASDAQ&symbol=MSFT")
#


@responses.activate
def test_get_demo_company():
    ##
    purge_cache()
    clear_symbols()
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    responses._add_from_file(file_path=data_path("msft-transcript-response.yaml"))
    responses._add_from_file(file_path=data_path("msft-company-events.yaml"))
    ##
    company = get_company("msft")
    events = company.events()
    ##
    assert len(events) == 20
    assert events[0].year == 2024
    assert events[0].quarter == 3



import argparse
import os

import earningscall  # noqa: F401
from earningscall import get_company
from earningscall.utils import configure_sane_logging

# TODO: Set your API key here:
# earningscall.api_key = "YOUR SECRET API KEY GOES HERE"


parser = argparse.ArgumentParser(description='')
parser.add_argument('--debug', action='store_true', help='Enable debug logs')
parser.add_argument('--sp-500', action='store_true', help='Show S&P500 Companies')

args = parser.parse_args()
configure_sane_logging()


directory = "audio_files"
os.makedirs(directory, exist_ok=True)


file_name = os.path.join(
    directory,
    "NASDAQ_META_2024_Q3.mp3",
)
company = get_company("meta")  # Lookup Meta, Inc by its ticker symbol, "META"
company.download_audio_file(year=2024, quarter=3, file_name=file_name)



from earningscall.api import get_user_agent


def test_get_user_agent():
    user_agent = get_user_agent()
    assert user_agent is not None
    assert "EarningsCallPython/" in user_agent
    assert "Python" in user_agent
    assert "Requests" in user_agent



import requests
import responses


# NOTE: this test is only testing the behavior of the responses library, and therefore it is not testing any
#       code in earningscall at all.


@responses.activate
def test_load_sp500_tickers():
    ##
    responses.add(
        responses.GET,
        "http://www.example.com",
        body="first response",
        status=200,
    )
    responses.add(
        responses.GET,
        "http://www.example.com",
        body="second response",
        status=200,
    )
    ##
    first_response = requests.get("http://www.example.com").text
    assert first_response == "first response"
    second_response = requests.get("http://www.example.com").text
    assert second_response == "second response"
    # It just returns last response over and over again.
    second_response = requests.get("http://www.example.com").text
    assert second_response == "second response"
    second_response = requests.get("http://www.example.com").text
    assert second_response == "second response"
    ##



import importlib.metadata
import logging
import os
import platform
import time
import urllib.parse
from typing import Dict, Optional, Union

import requests
from requests_cache import CachedSession

import earningscall
from earningscall.errors import InvalidApiKeyError

log = logging.getLogger(__file__)

DOMAIN = os.environ.get("EARNINGSCALL_DOMAIN", "earningscall.biz")
API_BASE = f"https://v2.api.{DOMAIN}"
DEFAULT_RETRY_STRATEGY: Dict[str, Union[str, int, float]] = {
    "strategy": "exponential",
    "base_delay": 1,
    "max_attempts": 10,
}


def get_api_key():
    if earningscall.api_key:
        return earningscall.api_key
    e_call_key = os.environ.get("ECALL_API_KEY")
    earnings_call_key = os.environ.get("EARNINGSCALL_API_KEY")
    if e_call_key:
        return e_call_key
    if earnings_call_key:
        return earnings_call_key
    return "demo"


def api_key_param():
    return {"apikey": get_api_key()}


def is_demo_account():
    return get_api_key() == "demo"


def cache_session() -> CachedSession:
    return CachedSession(
        ".earningscall_cache",
        backend="sqlite",
        cache_control=True,
        use_temp=True,
        ignored_parameters=[],
    )


def cached_urls():
    return cache_session().cache.urls()


def purge_cache():
    return cache_session().cache.clear()


def get_earnings_call_version():
    try:
        return importlib.metadata.version("earningscall")
    except importlib.metadata.PackageNotFoundError:
        return None


def get_user_agent():
    sdk_name = "EarningsCallPython"
    sdk_version = get_earnings_call_version()
    python_version = platform.python_version()
    os_info = f"{platform.system()} {platform.release()}"
    arch = platform.machine()
    requests_version = requests.__version__
    user_agent = f"{sdk_name}/{sdk_version} (Python/{python_version}; {os_info}; {arch}) Requests/{requests_version}"
    return user_agent


def get_headers():
    earnings_call_version = get_earnings_call_version()
    return {
        "User-Agent": get_user_agent(),
        "X-EarningsCall-Version": earnings_call_version,
    }


def can_retry(response: requests.Response) -> bool:
    if response.status_code == 429:
        return True
    # Check for 5XX errors
    if response.status_code >= 500 and response.status_code < 600:
        return True
    return False


def is_success(response: requests.Response) -> bool:
    # TODO: Do we need to check for 2xx status codes?
    return response.status_code == 200


def do_get(
    path: str,
    use_cache: bool = False,
    **kwargs,
) -> requests.Response:
    """
    Do a GET request to the API with exponential backoff retry for rate limits.

    Args:
        path (str): The path to request.
        use_cache (bool): Whether to use the cache.
        **kwargs: Additional arguments to pass to the request.

    Returns:
        requests.Response: The response from the API.
    """
    params = {
        **api_key_param(),
        **kwargs.get("params", {}),
    }
    url = f"{API_BASE}/{path}"
    if log.isEnabledFor(logging.DEBUG):
        full_url = f"{url}?{urllib.parse.urlencode(params)}"
        log.debug(f"GET: {full_url}")

    retry_strategy = earningscall.retry_strategy or DEFAULT_RETRY_STRATEGY
    delay = retry_strategy["base_delay"]
    max_attempts = int(retry_strategy["max_attempts"])

    for attempt in range(max_attempts):
        if use_cache and earningscall.enable_requests_cache:
            response = cache_session().get(url, params=params)
        else:
            response = requests.get(
                url,
                params=params,
                headers=get_headers(),
                stream=kwargs.get("stream"),
            )

        if is_success(response):
            return response

        if response.status_code == 401:
            raise InvalidApiKeyError(
                "Your API key is invalid. You can get your API key at: https://earningscall.biz/api-key"
            )

        if not can_retry(response):
            return response

        if attempt < max_attempts - 1:  # Don't sleep after the last attempt
            if retry_strategy["strategy"] == "exponential":
                wait_time = delay * (2**attempt)  # Exponential backoff: 1s -> 2s -> 4s -> 8s -> 16s -> 32s -> 64s
            elif retry_strategy["strategy"] == "linear":
                wait_time = delay * (attempt + 1)  # Linear backoff: 1s -> 2s -> 3s -> 4s -> 5s -> 6s -> 7s
            else:
                raise ValueError("Invalid retry strategy. Must be one of: 'exponential', 'linear'")
            # TODO: Should we log a warning here?  Does the customer want to see this log?
            log.warning(
                f"Rate limited (429). Retrying in {wait_time} seconds... (Attempt {attempt + 1}/{max_attempts})"
            )
            time.sleep(float(wait_time))

    return response  # Return the last response if all retries failed


def get_calendar_api_operation(year: int, month: int, day: int) -> dict:
    params = {
        "year": str(year),
        "month": str(month),
        "day": str(day),
    }
    response = do_get("calendar", params=params)
    response.raise_for_status()
    return response.json()


def get_events(exchange: str, symbol: str) -> Optional[dict]:
    params = {
        "exchange": exchange,
        "symbol": symbol,
    }
    response = do_get("events", params=params)
    if response.status_code != 200:
        return None
    return response.json()


def get_transcript(
    exchange: str,
    symbol: str,
    year: int,
    quarter: int,
    level: Optional[int] = None,
) -> Optional[dict]:
    """
    Get the transcript for a given exchange, symbol, year, and quarter.

    :param str exchange: The exchange to get the transcript for.
    :param str symbol: The symbol to get the transcript for.
    :param int year: The year to get the transcript for.
    :param int quarter: The quarter to get the transcript for.
    :param Optional[int] level: The level to get the transcript for.

    :return: The transcript for the given exchange, symbol, year, and quarter.
    """
    params = {
        "exchange": exchange,
        "symbol": symbol,
        "year": str(year),
        "quarter": str(quarter),
        "level": str(level or 1),
    }
    response = do_get("transcript", params=params)
    response.raise_for_status()
    return response.json()


def get_symbols_v2():
    response = do_get("symbols-v2.txt", use_cache=True)
    if response.status_code != 200:
        return None
    return response.text


def get_sp500_companies_txt_file():
    response = do_get("symbols/sp500.txt", use_cache=True)
    if response.status_code != 200:
        return None
    return response.text


def download_audio_file(
    exchange: str,
    symbol: str,
    year: int,
    quarter: int,
    file_name: Optional[str] = None,
) -> Optional[str]:
    """
    Get the audio for a given exchange, symbol, year, and quarter.

    :param str exchange: The exchange to get the audio for.
    :param str symbol: The symbol to get the audio for.
    :param int year: The 4-digit year to get the audio for.
    :param int quarter: The quarter to get the audio for (1, 2, 3, or 4).
    :param file_name: Optionally specify the filename to save the audio to.
    :return: The filename of the downloaded audio file.
    :rtype Optional[str]: The filename of the downloaded audio file.
    """
    params = {
        "exchange": exchange,
        "symbol": symbol,
        "year": str(year),
        "quarter": str(quarter),
    }
    local_filename = file_name or f"{exchange}_{symbol}_{year}_{quarter}.mp3"
    with do_get("audio", params=params, stream=True) as response:
        response.raise_for_status()
        with open(local_filename, "wb") as f:
            for chunk in response.iter_content(chunk_size=8192):
                f.write(chunk)
    return local_filename


def download_slide_deck(
    exchange: str,
    symbol: str,
    year: int,
    quarter: int,
    file_name: Optional[str] = None,
) -> Optional[str]:
    """
    Get the slide deck for a given exchange, symbol, year, and quarter.

    :param str exchange: The exchange to get the slide deck for.
    :param str symbol: The symbol to get the slide deck for.
    :param int year: The 4-digit year to get the slide deck for.
    :param int quarter: The quarter to get the slide deck for (1, 2, 3, or 4).
    :param file_name: Optionally specify the filename to save the slide deck to.
    :return: The filename of the downloaded slide deck file.
    :rtype Optional[str]: The filename of the downloaded slide deck file.
    """
    params = {
        "exchange": exchange,
        "symbol": symbol,
        "year": str(year),
        "quarter": str(quarter),
    }
    local_filename = file_name or f"{exchange}_{symbol}_{year}_{quarter}_slides.pdf"
    with do_get("slides", params=params, stream=True) as response:
        response.raise_for_status()
        with open(local_filename, "wb") as f:
            for chunk in response.iter_content(chunk_size=8192):
                f.write(chunk)
    return local_filename



#!/usr/bin/env python3
"""
Script to download slides for a given company and quarter
"""

import earningscall


def test_slides_api():
    """Test the new slides API functionality"""

    # Get a company
    company = earningscall.get_company("MSFT")
    if not company:
        print("Could not find MSFT company")
        return

    print(f"Testing slides API for {company.company_info.name}")

    try:
        # Try to download slides for Q1 2025 (using demo data)
        slide_file = company.download_slide_deck(year=2025, quarter=1)
        if slide_file:
            print(f"Successfully downloaded slides: {slide_file}")
        else:
            print("No slides available for the requested quarter")

    except Exception as e:
        print(f"Error downloading slides: {e}")
        # This is expected for demo account or if slides aren't available


if __name__ == "__main__":
    test_slides_api()



from datetime import date
import earningscall  # noqa: F401

from earningscall import get_calendar

# TODO: Set your API key here:
# earningscall.api_key = "YOUR API KEY HERE"

events = get_calendar(date(2025, 1, 10))

for event in events:
    print(
        f"{event.company_name} - Q{event.quarter} {event.year} on: {event.conference_date.astimezone().isoformat()} Transcript Ready: {event.transcript_ready}"
    )



import argparse
import logging

from earningscall import get_all_companies, get_sp500_companies
from earningscall.utils import configure_sane_logging

parser = argparse.ArgumentParser(description='')
parser.add_argument('--debug', action='store_true', help='Enable debug logs')
parser.add_argument('--sp-500', action='store_true', help='Show S&P500 Companies')

args = parser.parse_args()
level = logging.DEBUG
if args.debug:
    level = logging.DEBUG
configure_sane_logging(level=level)

if args.sp_500:
    get_func = get_sp500_companies
else:
    get_func = get_all_companies

for company in get_func():
    print(f"{company.company_info} -- {company.company_info.sector} -- {company.company_info.industry}")



import pytest
import responses

import earningscall
from earningscall import get_all_companies, get_company, get_sp500_companies
from earningscall.api import purge_cache
from earningscall.symbols import clear_symbols
from earningscall.utils import data_path


@pytest.fixture(autouse=True)
def run_before_and_after_tests():
    """Fixture to execute asserts before and after a test is run"""
    # Setup: fill with any logic you want
    earningscall.api_key = None
    purge_cache()
    clear_symbols()
    yield  # this is where the testing happens
    # Teardown : fill with any logic you want
    earningscall.api_key = None


@responses.activate
def test_get_company_not_found():
    ##
    earningscall.api_key = "foobar"  # Bogus key to avoid check for "demo"
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    ##
    company = get_company("blah blah blah")
    ##
    assert company is None


@responses.activate
def test_get_all_companies():
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    ##
    companies = [company for company in get_all_companies()]
    ##
    # for company in get_all_companies():
    #     print(company)
    assert len(companies) == 5807
    assert str(companies[0]) == "Agilent Technologies, Inc."
    assert str(companies[1]) == "Alcoa Corporation"
    assert str(companies[-1]) == "Zynex, Inc."


@responses.activate
def test_get_sp_500_companies():
    responses._add_from_file(file_path=data_path("sp500-company-list.yaml"))
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    ##
    earningscall.api_key = "foobar"  # Bogus key to avoid check for "demo"
    companies = [company for company in get_sp500_companies()]
    ##
    assert len(companies) == 493
    assert str(companies[0]) == "Apple Inc."
    assert str(companies[1]) == "Microsoft Corporation"
    assert str(companies[-1]) == "News Corporation"


@responses.activate
def test_get_sp_500_companies_failed_request():
    responses._add_from_file(file_path=data_path("sp500-company-list-failed.yaml"))
    ##
    earningscall.api_key = "foobar"  # Bogus key to avoid check for "demo"
    companies = [company for company in get_sp500_companies()]
    ##
    assert len(companies) == 0



import pytest
import responses

import earningscall
from earningscall.api import API_BASE, purge_cache
from earningscall.symbols import Symbols, CompanyInfo
from earningscall.utils import data_path


@pytest.fixture(autouse=True)
def run_before_and_after_tests(tmpdir):
    """Fixture to execute asserts before and after a test is run"""
    # Setup: fill with any logic you want
    earningscall.api_key = None
    purge_cache()
    yield  # this is where the testing happens
    # Teardown : fill with any logic you want
    earningscall.api_key = None


@responses.activate
def test_load_symbols_txt_v2():
    ##
    responses.patch(API_BASE)
    responses._add_from_file(file_path=data_path("symbols-v2.yaml"))
    ##
    symbols = Symbols.load_txt_v2()
    ##
    assert len(symbols) == 5807
    _symbol = symbols.get_exchange_symbol("NASDAQ_AAPL")
    assert _symbol.name == "Apple Inc."
    assert _symbol.exchange == "NASDAQ"
    assert _symbol.sector == "Technology"
    assert _symbol.industry == "Consumer Electronics"
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == "https://v2.api.earningscall.biz/symbols-v2.txt?apikey=demo"
    ##
    symbols.remove_exchange_symbol("NASDAQ_AAPL")
    with pytest.raises(KeyError):
        symbols.get_exchange_symbol("NASDAQ_AAPL")


@responses.activate
def test_load_symbols_txt_v2_missing_edge_cases():
    ##
    responses.patch(API_BASE)
    responses._add_from_file(file_path=data_path("symbols-v2-missing-edge-cases.yaml"))
    ##
    symbols = Symbols.load_txt_v2()
    ##
    assert len(symbols) == 4
    _symbol = symbols.get_exchange_symbol("UNKNOWN_A")
    assert _symbol.name == "Agilent Technologies, Inc."
    assert _symbol.exchange == "UNKNOWN"
    assert _symbol.sector == "Unknown"
    assert _symbol.industry == "Unknown"
    _symbol = symbols.get_exchange_symbol("NASDAQ_AACG")
    assert _symbol.name == "ATA Creativity Global"
    assert _symbol.exchange == "NASDAQ"
    assert _symbol.sector == "Consumer Defensive"
    assert _symbol.industry == "Education & Training Services"
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == "https://v2.api.earningscall.biz/symbols-v2.txt?apikey=demo"


def test_symbols_serialization_to_text_v2():
    ##
    _symbols = Symbols()
    _symbols.add(
        CompanyInfo(
            exchange="TSX",
            symbol="TLRY",
            name="Tilray, Inc",
            sector="Energy",
            industry="Electronic Gaming & Multimedia",
        )
    )
    _symbols.add(
        CompanyInfo(
            exchange="TSX",
            symbol="ACB",
            name="Aurora Cannabis Inc.",
            sector="Technology",
            industry="Electronic Gaming & Multimedia",
        )
    )
    _symbols.add(
        CompanyInfo(
            exchange="NASDAQ",
            symbol="HITI",
            name="High Tide Inc.",
            sector="Consumer Cyclical",
            industry="Electronic Gaming & Multimedia",
        )
    )
    ##
    result = _symbols.to_txt_v2()
    ##
    _deserialized_symbols = Symbols.from_txt_v2(result)
    assert len(_deserialized_symbols) == 3
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").name == "Tilray, Inc"
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").exchange == "TSX"
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").sector == "Energy"
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").industry == "Electronic Gaming & Multimedia"
    assert _deserialized_symbols.get_exchange_symbol("TSX_ACB").name == "Aurora Cannabis Inc."
    assert _deserialized_symbols.get_exchange_symbol("TSX_ACB").sector == "Technology"
    assert _deserialized_symbols.get_exchange_symbol("TSX_ACB").industry == "Electronic Gaming & Multimedia"
    assert _deserialized_symbols.get_exchange_symbol("NASDAQ_HITI").name == "High Tide Inc."


def test_symbols_serialization_v1():
    _symbols = Symbols()
    _symbols.add(CompanyInfo(exchange="TSX", symbol="TLRY", name="Tilray, Inc", sector="Energy", industry="Uranium"))
    _symbols.add(
        CompanyInfo(exchange="TSX", symbol="ACB", name="Aurora Cannabis Inc.", sector="Energy", industry="Uranium")
    )
    _symbols.add(
        CompanyInfo(exchange="NASDAQ", symbol="HITI", name="High Tide Inc.", sector="Energy", industry="Uranium")
    )
    ##
    result = _symbols.to_json()
    ##
    _deserialized_symbols = Symbols.from_json(result)
    assert len(_deserialized_symbols) == 3
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").name == "Tilray, Inc"
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").sector == "Energy"
    assert _deserialized_symbols.get_exchange_symbol("TSX_TLRY").industry == "Uranium"
    assert _deserialized_symbols.get_exchange_symbol("TSX_ACB").name == "Aurora Cannabis Inc."
    assert _deserialized_symbols.get_exchange_symbol("NASDAQ_HITI").name == "High Tide Inc."


def test_exchanges_in_order():
    assert earningscall.symbols.EXCHANGES_IN_ORDER == [
        "NYSE",
        "NASDAQ",
        "AMEX",
        "TSX",
        "TSXV",
        "OTC",
        "LSE",
        "CBOE",
        "STO",
    ]



from datetime import datetime
import os

import earningscall  # noqa: F401
from earningscall import get_sp500_companies
from earningscall.company import Company

# TODO: Set your API key here:
# earningscall.api_key = "YOUR SECRET API KEY GOES HERE"

directory = "data/transcript_texts"
os.makedirs(directory, exist_ok=True)


def download_transcript_texts(company: Company):
    print(f"Downloading all transcript texts for: {company}..")
    for event in company.events():
        if datetime.now().timestamp() < event.conference_date.timestamp():
            print(
                f"* {company.company_info.symbol} Q{event.quarter} {event.year} -- skipping, conference date in the future"
            )
            continue
        file_name = os.path.join(
            directory,
            f"{company.company_info.exchange}_{company.company_info.symbol}_{event.year}_Q{event.quarter}.text",
        )
        if os.path.exists(file_name):
            print(f"* {company.company_info.symbol} Q{event.quarter} {event.year} -- already downloaded")
        else:
            print(f"* Downloading transcript text for {company.company_info.symbol} Q{event.quarter} {event.year}...")
            transcript = company.get_transcript(event=event)
            if transcript:
                # Save transcript text to file
                with open(file_name, "w") as fd:
                    fd.write(transcript.text)
                print(
                    f" Downloaded transcript text for {company.company_info.symbol} Q{event.quarter} {event.year} to: {file_name}"
                )
            else:
                print(f" No transcript text found for {company.company_info.symbol} Q{event.quarter} {event.year}")


for company in get_sp500_companies():
    download_transcript_texts(company)



class BaseError(RuntimeError):
    """
    Base error
    """

    def __init__(self, msg=None):
        self.msg = msg

    def __str__(self):
        if self.msg:
            return str(self.msg)
        return ""


class ClientError(BaseError):
    """
    Used to return 4XX errors.
    """

    status: int = 400  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/400


class InsufficientApiAccessError(ClientError):
    pass


class InvalidApiKeyError(ClientError):
    pass



from earningscall.sectors import index_to_sector, index_to_industry, sector_to_index, industry_to_index


def test_unknown_sector():
    assert sector_to_index("Bad Sector") == -1


def test_unknown_sector_index():
    assert index_to_sector(-1) == "Unknown"


def test_unknown_industry():
    assert industry_to_index("Bad Industry") == -1


def test_unknown_industry_index():
    assert index_to_industry(-1) == "Unknown"



import json
import logging
from collections import defaultdict
from typing import Optional, Iterator, List

from earningscall.api import get_symbols_v2, is_demo_account
from earningscall.errors import InsufficientApiAccessError
from earningscall.sectors import sector_to_index, industry_to_index, index_to_sector, index_to_industry

# WARNING: Add new indexes to the *END* of this list
EXCHANGES_IN_ORDER = [
    "NYSE",
    "NASDAQ",
    "AMEX",
    "TSX",
    "TSXV",
    "OTC",
    "LSE",
    "CBOE",
    "STO",
]

log = logging.getLogger(__file__)


def exchange_to_index(_exchange: Optional[str]) -> int:
    if not _exchange:
        return -1
    try:
        return EXCHANGES_IN_ORDER.index(_exchange)
    except ValueError:
        return -1


def index_to_exchange(_index: int) -> str:
    if _index == -1:
        return "UNKNOWN"
    try:
        return EXCHANGES_IN_ORDER[_index]
    except IndexError:
        return "UNKNOWN"


security_type_pattern = {
    "NASDAQ": r" - .*$",
    "NYSE": r" (Common Stock|Warrants)$",
    "AMEX": r" (Common Stock|Warrants)$",
}


class CompanyInfo:

    exchange: Optional[str]
    symbol: Optional[str]
    name: Optional[str]
    security_name: Optional[str]
    sector: Optional[str]
    industry: Optional[str]

    def __init__(self, **kwargs):
        self.exchange = None
        self.symbol = None
        self.name = None
        self.sector = None
        self.industry = None
        for key, value in kwargs.items():
            self.__setattr__(key, value)

    def __str__(self):
        return f"({self.exchange}: {self.symbol} - {self.name})"

    def to_json(self):
        return json.dumps(self.__dict__)

    def to_txt_row(self):
        return [
            str(exchange_to_index(self.exchange)),
            self.symbol,
            self.name,
        ]

    def to_txt_v2_row(self):
        return [
            str(exchange_to_index(self.exchange)),
            self.symbol,
            self.name,
            str(sector_to_index(self.sector)),
            str(industry_to_index(self.industry)),
        ]

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __hash__(self):
        k = self.__dict__.keys()
        sorted(k)
        return sum(map(lambda x: f"{x}-{self.__dict__[x]}".__hash__(), k))

    def exchange_symbol(self):
        return f"{self.exchange}_{self.symbol}"


class Symbols:

    def __init__(self):
        self.exchanges = set()
        self.by_name = defaultdict(set)
        self.by_exchange_and_sym = {}

    def add(self, _sym: CompanyInfo):
        # size_before = len(self.by_exchange_and_sym)
        self.exchanges.add(_sym.exchange)
        self.by_name[_sym.name].add(_sym)
        self.by_exchange_and_sym[f"{_sym.exchange}_{_sym.symbol}"] = _sym
        # if len(self.by_exchange_and_sym) == size_before:
        #     log.debug(f"Duplicate: {_sym}")

    def get_all(self) -> Iterator[CompanyInfo]:
        for _exchange_symbol, _symbol in self.by_exchange_and_sym.items():
            yield _symbol

    def get(self, _exchange: str, _symbol: str) -> CompanyInfo:
        return self.get_exchange_symbol(f"{_exchange}_{_symbol}")

    def get_exchange_symbol(self, exchange_symbol: str) -> CompanyInfo:
        return self.by_exchange_and_sym[exchange_symbol]

    def lookup_company(self, symbol: str, exchange: Optional[str] = None) -> Optional[CompanyInfo]:
        if exchange:
            return self.get(exchange, symbol.upper())
        for exchange in EXCHANGES_IN_ORDER:
            try:
                _symbol = self.get(exchange, symbol.upper())
                if _symbol:
                    return _symbol
            except KeyError:
                pass
        if is_demo_account():
            raise InsufficientApiAccessError(
                f"\"{symbol}\" requires an API Key for access.  To get your API Key,"
                f" see: https://earningscall.biz/api-pricing"
            )
        return None

    def remove_exchange_symbol(self, exchange_symbol: str):
        _symbol = self.by_exchange_and_sym[exchange_symbol]
        del self.by_name[_symbol.name]
        del self.by_exchange_and_sym[exchange_symbol]

    @staticmethod
    def remove_keys(symbol_as_dict: dict, keys_to_remove: set):
        return {key: value for key, value in symbol_as_dict.items() if key not in keys_to_remove}

    def without_security_names(self) -> List[dict]:
        return [
            self.remove_keys(symbol_as_dict, {"security_name", "sector", "industry"})
            for symbol_as_dict in self.to_dicts()
        ]

    def to_dicts(self) -> List[dict]:
        return [__symbol.__dict__ for __symbol in self.get_all()]

    def to_json(self, remove_security_names: bool = False) -> str:
        if remove_security_names:
            return json.dumps(self.without_security_names())
        return json.dumps(self.to_dicts())

    def to_txt_v2(self) -> str:
        exchange_symbol_names = [__symbol.to_txt_v2_row() for __symbol in self.get_all()]
        sorted_rows = sorted(exchange_symbol_names, key=lambda row: row[1])
        return "\n".join(["\t".join(row) for row in sorted_rows])

    @staticmethod
    def from_json(json_str):
        __symbols = Symbols()
        for item in json.loads(json_str):
            __symbols.add(CompanyInfo(**item))
        return __symbols

    @staticmethod
    def from_txt_v2(txt_str):
        __symbols = Symbols()
        for line in txt_str.split("\n"):
            _exchange_index, _symbol, _name, _sector_index, _industry_index = line.split("\t")
            __symbols.add(
                CompanyInfo(
                    exchange=index_to_exchange(int(_exchange_index)),
                    symbol=_symbol,
                    name=_name,
                    sector=index_to_sector(int(_sector_index)),
                    industry=index_to_industry(int(_industry_index)),
                )
            )
        return __symbols

    @staticmethod
    def load_txt_v2():
        return Symbols.from_txt_v2(get_symbols_v2())

    def __iter__(self):
        for _exchange_symbol, _symbol in self.by_exchange_and_sym.items():
            yield _symbol

    def __len__(self):
        return len(self.by_exchange_and_sym)


def load_symbols() -> Symbols:
    return Symbols.load_txt_v2()


_symbols = None


def get_symbols() -> Symbols:
    global _symbols
    if not _symbols:
        _symbols = load_symbols()
    return _symbols


def clear_symbols():
    global _symbols
    _symbols = None



import logging
import os
import pathlib
import sys

import earningscall


def project_file_path(base_dir, file_name):
    result = os.path.join(pathlib.Path(earningscall.__file__).resolve().parent.parent, base_dir)
    if file_name is None:
        return result
    return os.path.join(result, file_name)


def data_path(file_name=None):
    return project_file_path("tests/data", file_name)


def enable_debug_logs():
    logging.basicConfig(level=logging.DEBUG)


LOG_FORMAT = "[%(asctime)s] [%(levelname)8s] --- %(message)s (%(filename)s:%(lineno)s)"


def configure_sane_logging(level=logging.DEBUG):
    # Manually set the following loggers to "INFO", since they tend to be VERY noisy, and when we are
    # debugging our own application, we only care about DEBUG logs from OUR application, not these libs.
    for name in ['urllib3']:
        logging.getLogger(name).setLevel(logging.INFO)
    logging.getLogger("filelock").setLevel(logging.WARNING)
    logging.basicConfig(level=level, stream=sys.stdout, format=LOG_FORMAT)



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from dataclasses_json import config
from dataclasses_json import dataclass_json
from marshmallow import fields


@dataclass_json
@dataclass
class CalendarEvent:
    """
    CalendarEvent
    """

    company_name: str
    exchange: str
    symbol: str
    year: int
    quarter: int
    transcript_ready: bool
    conference_date: Optional[datetime] = field(
        default=None,
        metadata=config(
            encoder=lambda date: date.isoformat() if date else None,
            decoder=lambda date: datetime.fromisoformat(date) if date else None,
            mm_field=fields.DateTime(format="iso"),
        ),
    )



from typing import Dict, Optional, Union

from earningscall.exports import get_company, get_all_companies, get_sp500_companies, get_calendar
from earningscall.symbols import Symbols, load_symbols

api_key: Optional[str] = None
enable_requests_cache: bool = True
retry_strategy: Optional[Dict[str, Union[str, int, float]]] = None

__all__ = [
    "get_company",
    "get_all_companies",
    "get_sp500_companies",
    "Symbols",
    "load_symbols",
    "get_calendar",
]



import logging

log = logging.getLogger(__file__)
sectors_file_name = "sectors.json"


SECTORS_IN_ORDER = [
    'Basic Materials',
    'Communication Services',
    'Consumer Cyclical',
    'Consumer Defensive',
    'Energy',
    'Financial Services',
    'Healthcare',
    'Industrials',
    'Real Estate',
    'Technology',
    'Utilities',
]


INDUSTRIES_IN_ORDER = [
    'Advertising Agencies',
    'Aerospace & Defense',
    'Agricultural Inputs',
    'Airlines',
    'Airports & Air Services',
    'Aluminum',
    'Apparel Manufacturing',
    'Apparel Retail',
    'Asset Management',
    'Auto & Truck Dealerships',
    'Auto Manufacturers',
    'Auto Parts',
    'Banks - Diversified',
    'Banks - Regional',
    'Beverages - Brewers',
    'Beverages - Non-Alcoholic',
    'Beverages - Wineries & Distilleries',
    'Biotechnology',
    'Broadcasting',
    'Building Materials',
    'Building Products & Equipment',
    'Business Equipment & Supplies',
    'Capital Markets',
    'Chemicals',
    'Coking Coal',
    'Communication Equipment',
    'Computer Hardware',
    'Confectioners',
    'Conglomerates',
    'Consulting Services',
    'Consumer Electronics',
    'Copper',
    'Credit Services',
    'Department Stores',
    'Diagnostics & Research',
    'Discount Stores',
    'Drug Manufacturers - General',
    'Drug Manufacturers - Specialty & Generic',
    'Education & Training Services',
    'Electrical Equipment & Parts',
    'Electronic Components',
    'Electronic Gaming & Multimedia',
    'Electronics & Computer Distribution',
    'Engineering & Construction',
    'Entertainment',
    'Farm & Heavy Construction Machinery',
    'Farm Products',
    'Financial Conglomerates',
    'Financial Data & Stock Exchanges',
    'Food Distribution',
    'Footwear & Accessories',
    'Furnishings, Fixtures & Appliances',
    'Gambling',
    'Gold',
    'Grocery Stores',
    'Health Information Services',
    'Healthcare Plans',
    'Home Improvement Retail',
    'Household & Personal Products',
    'Industrial Distribution',
    'Information Technology Services',
    'Infrastructure Operations',
    'Insurance - Diversified',
    'Insurance - Life',
    'Insurance - Property & Casualty',
    'Insurance - Reinsurance',
    'Insurance - Specialty',
    'Insurance Brokers',
    'Integrated Freight & Logistics',
    'Internet Content & Information',
    'Internet Retail',
    'Leisure',
    'Lodging',
    'Lumber & Wood Production',
    'Luxury Goods',
    'Marine Shipping',
    'Medical Care Facilities',
    'Medical Devices',
    'Medical Distribution',
    'Medical Instruments & Supplies',
    'Metal Fabrication',
    'Mortgage Finance',
    'Oil & Gas Drilling',
    'Oil & Gas E&P',
    'Oil & Gas Equipment & Services',
    'Oil & Gas Integrated',
    'Oil & Gas Midstream',
    'Oil & Gas Refining & Marketing',
    'Other Industrial Metals & Mining',
    'Other Precious Metals & Mining',
    'Packaged Foods',
    'Packaging & Containers',
    'Paper & Paper Products',
    'Personal Services',
    'Pharmaceutical Retailers',
    'Pollution & Treatment Controls',
    'Publishing',
    'REIT - Diversified',
    'REIT - Healthcare Facilities',
    'REIT - Hotel & Motel',
    'REIT - Industrial',
    'REIT - Mortgage',
    'REIT - Office',
    'REIT - Residential',
    'REIT - Retail',
    'REIT - Specialty',
    'Railroads',
    'Real Estate - Development',
    'Real Estate - Diversified',
    'Real Estate Services',
    'Recreational Vehicles',
    'Rental & Leasing Services',
    'Residential Construction',
    'Resorts & Casinos',
    'Restaurants',
    'Scientific & Technical Instruments',
    'Security & Protection Services',
    'Semiconductor Equipment & Materials',
    'Semiconductors',
    'Shell Companies',
    'Silver',
    'Software - Application',
    'Software - Infrastructure',
    'Solar',
    'Specialty Business Services',
    'Specialty Chemicals',
    'Specialty Industrial Machinery',
    'Specialty Retail',
    'Staffing & Employment Services',
    'Steel',
    'Telecom Services',
    'Textile Manufacturing',
    'Thermal Coal',
    'Tobacco',
    'Tools & Accessories',
    'Travel Services',
    'Trucking',
    'Uranium',
    'Utilities - Diversified',
    'Utilities - Independent Power Producers',
    'Utilities - Regulated Electric',
    'Utilities - Regulated Gas',
    'Utilities - Regulated Water',
    'Utilities - Renewable',
    'Waste Management',
]


def sector_to_index(_sector: str) -> int:
    try:
        return SECTORS_IN_ORDER.index(_sector)
    except ValueError:
        return -1


def index_to_sector(_index: int) -> str:
    if _index == -1:
        return "Unknown"
    try:
        return SECTORS_IN_ORDER[_index]
    except IndexError:
        return "Unknown"


def index_to_industry(_index: int) -> str:
    if _index == -1:
        return "Unknown"
    try:
        return INDUSTRIES_IN_ORDER[_index]
    except IndexError:
        return "Unknown"


def industry_to_index(_industry: str) -> int:
    try:
        return INDUSTRIES_IN_ORDER.index(_industry)
    except ValueError:
        return -1
